#include <iostream>
#include <sstream>
#include <iomanip>
#include <vector>
#include <unistd.h>
#include "pvo-metadata.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"

namespace PVO
{

  namespace Meta
  {

    void Metadata::define(FITS::PSRFits fitsfile)
    {

      this->filename = fitsfile.getFITSname();

      std::string file_loc = "";
      if(this->filename[0]!='/'){
	this->fileloc = get_current_dir_name();
	this->fileloc += "/";
      }
      this->fileloc += this->filename;

      fitsfile.gotoPHU();

      if(fitsfile.getKeyword<std::string>("HDRVER",this->headerversion)!=0) 
	this->headerversion = nullString;
      this->headerversion = deblank(this->headerversion);

      if(fitsfile.getKeyword<std::string>("SRC_NAME",this->srcname)!=0) this->srcname = nullString;
      else this->srcname = deblank(this->srcname);
      if(this->srcname=="") this->srcname = nullString;

      if(fitsfile.getKeyword<std::string>("RA",this->raj)!=0) this->raj = nullString;
      else this->rajd = dmsToDec(this->raj) * 15.;

      if(fitsfile.getKeyword<std::string>("DEC",this->decj)!=0) this->decj = nullString;
      else this->decjd = dmsToDec(this->decj);

      if(fitsfile.getKeyword<std::string>("DATE-OBS",this->date)!=0) this->date = nullString;

      if(fitsfile.getKeyword<float>("OBSFREQ",this->obsfreq)!=0) this->obsfreq = nullFloat;
  
      if(fitsfile.getKeyword<float>("BMAJ",this->bmaj)==0){
	fitsfile.getKeyword<float>("BMIN",this->bmin);
	fitsfile.getKeyword<float>("BPA",this->bpa);
	this->beamsize = 0.5*(bmaj+bmin); 
      }
      else this->bmaj = this->bmin = this->bpa = this->beamsize = nullFloat;

      if(fitsfile.getKeyword<float>("SCANLEN",this->scanlen)!=0) this->scanlen = nullFloat;

      fitsfile.gotoHistory();
      if(fitsfile.readColumnValue("TBIN",1,&this->tbin)!=0) this->tbin = nullFloat;
      else this->tbin *= 1000.;
      if(!(this->tbin>0.)) this->tbin = nullFloat;

      if(fitsfile.readColumnValue("NCHAN",1,&this->nchan)!=0) this->nchan = nullInt;
      if(this->nchan == 0) this->nchan = nullInt;

      if(fitsfile.readColumnValue("CHAN_BW",1,&this->chanbw)!=0) this->chanbw = nullFloat;
      if(!(this->chanbw>0.)) this->chanbw = nullFloat;

    }

    bool Metadata::close(float radius)
    {
      if(this->angsep < 0.) return false;
      else return (this->angsep < (radius + 0.5*this->beamsize));
    }

    bool Metadata::close(float ra, float dec, float radius)
    {
      this->setAngSep(ra,dec);
      return (angsep < (radius+0.5*this->beamsize));
    }


    std::string floatToString(float f)
    {
      if(f > nullFloat){
	std::stringstream ss;
	ss << std::fixed << std::setprecision(fltwidth) << f;
	return ss.str();
      }
      else return nullString;
    }

    std::string intToString(int i)
    {
      if(i > nullInt){
	std::stringstream ss;
	ss << i;
	return ss.str();
      }
      else return nullString;
    }

    void printMetadataShort(std::ostream &theStream, Metadata data)
    {
      std::vector<Metadata> datalist;
      datalist.push_back(data);
      printMetadataShort(theStream, datalist);
    }

    void printMetadataShort(std::ostream &theStream, std::vector<Metadata> datalist)
    {
      int filenameSize = 8; // size of "FILENAME"
      unsigned int i;
      for(i=0;i<datalist.size();i++) 
	filenameSize = std::max(filenameSize,int(datalist[i].getFilename().size()));
    
      theStream << std::setfill('-')  
		<< std::setw(filenameSize+1+15+15+31+11+11) << "-";
      if(datalist[0].showSep()) theStream << std::setw(13) << "-";

      theStream << "\n" << std::setfill(' ') 
		<< std::setw(filenameSize) << "FILENAME"  << " "
		<< std::setw(14)           << "RAJ"       << " "
		<< std::setw(14)           << "DECJ"      << " "
		<< std::setw(10)           << "BEAM FWHM" << " "
		<< std::setw(10)           << "OBSFREQ"   << " "
		<< std::setw(30)           << "DATE-OBS";
      if(datalist[0].showSep())
	theStream  << " " << std::setw(12) << "OFFSET"    << "\n";
      else theStream << "\n";
     
      theStream << std::setw(filenameSize) << ""          << " "
		<< std::setw(14)           << "[deg]"     << " "
		<< std::setw(14)           << "[deg]"     << " "
		<< std::setw(10)           << "[deg]"     << " "
		<< std::setw(10)           << "[MHz]"     << " "
		<< std::setw(30)           << "";
      if(datalist[0].showSep())
	theStream << " " << std::setw(12)  << "[deg]" << "\n";
      else  theStream << "\n";

      theStream << std::setfill('-')  
		<< std::setw(filenameSize+1+15+15+31+11+11) << "-";
      if(datalist[0].showSep()) theStream << std::setw(13) << "-";
      theStream << "\n"<< std::setfill(' ') ;
    
      for(i=0;i<datalist.size();i++){

	theStream << std::setw(filenameSize) << datalist[i].getFilename() << " ";

	if(datalist[0].showSep())
	  theStream << std::setprecision(6)
		    << std::setw(14)         << datalist[i].getRaD()       << " "
		    << std::setw(14)         << datalist[i].getDecD()      << " ";
	else
	  theStream << std::setw(14)         << datalist[i].getRaS()       << " "
		    << std::setw(14)         << datalist[i].getDecS()      << " ";

	theStream << std::setprecision(4)
		  << std::setw(10)           << floatToString(datalist[i].getBeamsize()) << " "
		  << std::setw(10)           << floatToString(datalist[i].getObsfreq())  << " "
		  << std::setw(30)           << datalist[i].getDate();

	if(datalist[0].showSep()) 
	  theStream << " " << std::setw(12)  << floatToString(datalist[i].getAngSep())   << "\n";
	else theStream << "\n";

      }


    }

    void printMetadataFull(std::string filename, Metadata data)
    {
      std::vector<Metadata> datalist;
      datalist.push_back(data);
      printMetadataFull(filename, datalist);
    }

    void printMetadataFull(std::string filename, std::vector<Metadata> datalist)
    {
      std::ofstream fileout(filename.c_str());

      unsigned int i;
    
      fileout << "Filename,"
	      << "\"File Location\","
	      << "\"Header version\","
	      << "\"Source Name\","
	      << "RAJ,"
	      << "DECJ,"
	      << "RAJd,"
	      << "DECJd,"
	      << "BMAJ,"
	      << "BMIN,"
	      << "BPA,"
	      << "Date-Obs,"
	      << "Obsfreq,"
	      << "ChanBW,"
	      << "NChan,"
	      << "\"Scan length\","
	      << "Tbin";	    
      if(datalist[0].showSep())	fileout  << ",Offset\n";
      else fileout << "\n";
     
      for(i=0;i<datalist.size();i++){

	fileout << "\""<< datalist[i].getFilename()      << "\",";
	fileout << "\""<< datalist[i].getFileloc()       << "\",";
	fileout << "\""<< datalist[i].getHeaderVersion() << "\",";
	fileout << "\""<< datalist[i].getSrcname()       << "\",";
	
	fileout << "\""<< datalist[i].getRaS()  <<"\""      << ","
		<< "\""<< datalist[i].getDecS() <<"\""      << ",";
	fileout << std::fixed << std::setprecision(6)
		<< datalist[i].getRaD()                     << ","
		<< datalist[i].getDecD()                    << ",";

	fileout << floatToString(datalist[i].getBmaj())     << ","
		<< floatToString(datalist[i].getBmin())     << ","
		<< floatToString(datalist[i].getBpa())      << ","
		<< datalist[i].getDate()                    << ","
		<< floatToString(datalist[i].getObsfreq())  << ","
		<< floatToString(datalist[i].getChanbw())   << ","
		<< intToString(datalist[i].getNchan())      << ","
		<< floatToString(datalist[i].getScanlen())  << ","
		<< floatToString(datalist[i].getTbin());

	if(datalist[0].showSep()) 
	  fileout << "," << floatToString(datalist[i].getAngSep()) << "\n";
	else fileout << "\n";

      }

      fileout.close();

    }

  }// end namespace Meta

} // end namespace PVO
