#ifndef UTILS_HH_
#define UTILS_HH_
#include <iostream>
#include <string>
#include <vector>

namespace PVO
{

  /** Standard NULL value for strings. */
  const std::string nullString = "NULL";

  /** Standard NULL value for floats. */
  const float       nullFloat = -9999.;

  /** Standard NULL value for ints. */
  const int         nullInt = -9999;

  /** Help text for pvoconvert -h */
  const std::string pvoconvert_usage_text = 
"Usage: pvoconvert [options] <survey file>\n\
  Options:\n\
  -H  Use the PH (High Latitude) Survey settings, rather than those for the\n\
       PM/Multibeam Survey\n\
  -h  Display this help page.\n\
  -v  Verbose mode for error/warning messages.\n\
---------\n\n\
 PVOConvert converts pulsar survey data into PSRFITS format.\n\
 It can distinguish between the 70cm data and 21cm Multibeam data, but to\n\
  specify particular data as coming from the High Latitude survey rather\n\
  than the standard Multibeam survey, use the -h option.\n\
 Output files will be encoded according to the survey type and date & time\n\
  of observation.\n";
  /** Brief help text for pvoconvert -h */
  const std::string pvoconvert_usage_text_short = 
"Usage: pvoconvert [options] <survey file>\n\
  Options:\n\
  -H  Use the PH (High Latitude) Survey settings, rather than those for the\n\
       PM/Multibeam Survey\n\
  -h  Display the help page.\n\
  -v  Verbose mode for error/warning messages.\n";

  /** Help text for pvosummary -h */
  const std::string pvosummary_usage_text = 
"Usage:\n\
  1. pvosummary [options]\n\
  2. pvosummary [options] <FILE>\n\
---------\n\
 pvosummary  prints out some basic metadata for either all FITS files in the\n\
  current directory (example 1), or a single FITS file (example 2).\n\
 The parameters printed out are:\n\
   RA, DEC, BEAM FWHM, OBSFREQ, DATE-OBS\n\
 If a keyword is not present, "+nullString+" is printed.\n\
 The full set of parameters can be written in csv format to a file using the\n\
   -c or --csv option.\n\
 The list of parameters can be obtained by using the -p option.\n\
 FITS files are assumed to have suffixes of .fits or .rf\n\n\
 Options:\n\
 -c <file>    Write all parameters in csv format to the specified file.\n\
 --csv <file>\n\
 -p           List all parameters\n\
 -q           Quiet mode -- nothing printed to screen\n\
               (used in conjunction with -c)\n\
 -h           Print this help message.\n\
 --help \n\
---------\n\
";
  /** Brief help text for pvosummary -h */
  const std::string pvosummary_usage_text_short = 
"Usage:\n\
  1. pvosummary [options]\n\
  2. pvosummary [options] <FILE>\n\
 Options:\n\
 -c <file>    Write all parameters in csv format to the specified file.\n\
 --csv <file>\n\
 -p           List all parameters\n\
 -q           Quiet mode -- nothing printed to screen\n\
               (used in conjunction with -c)\n\
 -h           Print a help message.\n\
 --help\n";

  /** Help text for pvoconesearch -h */
  const std::string pvoconesearch_usage_text = 
"Usage:\n\
  1. pvoconesearch ra dec radius\n\
  2. pvoconesearch -h\n\
---------\n\
  1. Prints RA, DEC, BEAM FWHM, OBSFREQ, DATE-OBS, OFFSET for all FITS files\n\
     that lie within \"radius\" of the position given by ra, dec.\n\
     OFFSET is angular offset from query position.\n\
  2. Prints this help information.\n\n\
 The valid files are listed in order of increasing angular separation from\n\
  the query position.\n\
 The RA and DEC can be in the form hh:mm:ss.ss etc or in decimal degrees.\n\
 The search radius is in degrees. The beam size is also taken into account,\n\
  so the effective search radius becomes (radius + 0.5*beamFWHM).\n\
 Note that if a keyword is not present, NULL is printed.\n\
 FITS files are assumed to have suffixes of .fits or .rf\n";
  /** Brief help text for pvoconesearch -h */
  const std::string pvoconesearch_usage_text_short = 
"Usage:\n\
  1. pvoconesearch ra dec radius\n\
  2. pvoconesearch -h\n";

  /** Report a general error, along with the offending function. */
  void PVOError(std::string subroutine, std::string error);
  
  /** Return the filename of the defintion file of the PSRFITS format. */
  inline std::string getPSRFITSdefn(){ 
    return getenv("PSRFITSDEFN"); 

    //   std::cerr << "Using local PSRFITS definition! --> $PVO/templates/psrheader_20070209.fits\n"; 
    //   return "/DATA/SITAR_1/whi550/VO/PulsarVO/templates/psrheader_20070209.fits"; 
  };

  std::string removeLeadingBlanks(std::string s);
  ///< Remove blank spaces from start of a string.

  std::string deblank(std::string s);
  ///< Remove blank spaces from start and end of a string.  
   
  std::string charToBinary(char c);
  ///< Convert a byte to binary representation.

  std::string binaryToString(std::vector<bool> v);
  ///< Convert a bit-wise binary representation to a set of bytes. 

  unsigned int * binaryToInt(std::vector<bool> v);
  ///<Convert a series of bits into a series of bytes in decimal format.

  double dmsToDec(std::string input);   
  ///< Converts a string in the format +12:23:34.45 to a decimal angle
  ///in degrees.

  std::string decToDMS(double input, int secondPrecision, 
		       std::string separator);
  ///< Converts a decimal into a dd:mm:ss.ss format.

  /** Particular instance of decToDMS(double,int,std::string) with
      standard precision & separator */
  inline std::string decToDMS(double input){ return decToDMS(input,2,":"); };

  /** Find the angular separation of two sky positions.*/
  double angSep(std::string ra1, std::string dec1, 
		std::string ra2, std::string dec2);

  /** Find the angular separation of two sky positions.*/
  double angSep(double ra1, double dec1, double ra2, double dec2);

  std::string getIAUName(double ra, double dec);  
  ///< Create an IAU-style formatted name, based on the equatorial position.

  std::string addDay(std::string date);
  ///< A function to add a day onto the date.

  /** Makes the DATE-OBS header keyword in the format mandated by the
      FITS standard. */
  std::string makeDateObs(std::string date, std::string time);

}

#endif /*UTILS_HH_*/
